/**
 * Application bootstrap
 */
(function($) {
	'use strict';
	var wndw = $(window);
    mui.init();
	function startApp() {
		// Bind events for touchable elements
		bindEvents();
		// Setup default ajax settings
		$.ajaxSetup({
			timeout: 30000,  // 设置ajax超时时间为30秒
			cache: false,
			// http://stackoverflow.com/questions/12506897/is-safari-on-ios-6-caching-ajax-results
			headers: { "cache-control": "no-cache" }
		});
		// Statistic
		if (Util.statisticEnabled) Util.statistic.onAppStart(VERSION_NUMBER);
		// Initialize FX
		FX.initialize();
		// Adjust layout size
		Layout.initialize(wndw.width(), wndw.height());
		// Check version update
        Util.checkUpdateOrNewInstall(function(result) {
            Util.data('appStart',result);

            if (result.isUpdate) {
                // force user re-login
                UserProfile.setUserLogout();
            }
            Util.checkVersionUpdate();
            
            var _profile = Util.data('profile.user'), _patient = Util.data('cache.personalInfo');
            if( _profile ) {
                $.extend(UserProfile, _profile);
                UserProfile.loginState = true;
                if (_patient) {
                    for (var i = 0; i < UserProfile.accounts.length; i++) {
                        if (_patient.id === UserProfile.accounts[i].id) {
                            Util.data('cache.personalInfo', UserProfile.accounts[i]);

                            break;
                        }
                    }
                }
                initAccount();
            } else {
//               clearDataOfLogout();
                CACHE.ajaxCardTypes();
                UserProfile.clearUserAutoLogin();
            }
        });
        CACHE.cacheExpressInfo();
		ScrollPosition.clearScrollPosition();
		// Initialize app
		AppManager.initialize();
		// Initialize the File System
		//FSManager.initialize();

		//设置设备uuid
		if (APPVERSION !== 'wechat' && APPVERSION !== 'web') {
			FuckHttp.globalData.deviceId = device.uuid;
			DEVICE_UUID = device.uuid;
		}

        if(!cordova){
            return;
        }
        cordova.exec(
            function(winParam) {},
            function(error) {

            },
            "OnlineConsultationPlugin",
            "action_laucher_app",
            [
                APPVERSION, HOSPITAL_ID
            ]
        );
	}

	window.networkState = 'No network connection';
	function checkConnection() {
		var networkState = navigator.network.connection.type;
		var states = {};
		states[Connection.UNKNOWN] = 'Unknown connection';
		states[Connection.ETHERNET] = 'Ethernet connection';
		states[Connection.WIFI] = 'WiFi connection';
		states[Connection.CELL_2G] = 'Cell 2G connection';
		states[Connection.CELL_3G] = 'Cell 3G connection';
		states[Connection.CELL_4G] = 'Cell 4G connection';
		states[Connection.NONE] = 'No network connection';
		window.networkState = states[networkState];
		Util.networkStateChanged(window.networkState);
		// 如果开启了移动统计...
		if (Util.statisticEnabled) Util.statistic.onConnectionChanged();
	}

	var device_ready = false;
	var document_ready = false;
	var template_ready = false;
	function initApp() {
		if ((device_ready && document_ready && template_ready) || (document_ready && template_ready && !mobile_system)) {
			startApp();
		}
	}
	$(function() {
		Util.log('application start!');
		document_ready = true;
		if ($('#J_templateContainer').length > 0) {
			Util.log('从页面编译模板...');
			$.each(Views.tmpl, function(index, value) {
				Views.tmpl[index] = template.compile($('#template_' + index).html());
			});
			template_ready = true;
			$('#J_templateContainer').remove();
		} else {
			// 注意：使用这种方式，需要给Chrome浏览器加上--allow-file-access-from-files启动参数
			Util.log('从文件编译模板...');
			var _tmplArr = [];
			$.each(Views.tmpl, function(key, value) {
				_tmplArr.push([key, value]);
			});
			(function loadTmplComplie(tmplUrl) {
				if (!tmplUrl) {
					template_ready = true;
					initApp();
				} else {
					$.ajax({
						url: tmplUrl[1],
						type: 'POST',
						data: {
							rtime: Util.rtime()
						},
						dataType: 'text',
						success: function(data) {
							Views.tmpl[tmplUrl[0]] = template.compile(data);
							loadTmplComplie(_tmplArr.pop());
						},
						error: function() {
							Util.log('未找到模板文件' + tmplUrl[1]);
						}
					});
				}
			}) (_tmplArr.pop());
		}
		FastClick.attach(document.body);
		initApp();
	});
	document.addEventListener('deviceready', function() {
		device_ready = true;
		document.addEventListener('online', checkConnection, false);
		document.addEventListener('offline', checkConnection, false);
		checkConnection();
		initApp();
	}, false);
	document.addEventListener('resume', function() {
		Util.checkVersionUpdate();
	}, false);

	function bindEvents() {
		var hasTouch = 'ontouchstart' in window,
			RESIZE_EVENT = 'onorientationchange' in window ? 'orientationchange' : 'resize',
			TOUCH_EVENT = hasTouch ? 'touchstart touchend touchcancel touchmove' : 'mousedown mouseup mouseout mousemove',
			TOUCHDOWN_TARGET, TOUCH_LONGTAP_TID = null,
			triggerCallback = function(callbackName, btnId, event) {
				if (APPVERSION === 'web') {
					Callback[callbackName](btnId, event);  // let exception throw
				} else {
					try { Callback[callbackName](btnId, event); } catch (ignore) {}
				}
			},
			start_point, move_point;
		$(document.body).on(TOUCH_EVENT, '.ui-btn', function(e) {
			var _eType = e.type,
				_btn = $(this),
				_btnId = _btn.attr('id');
			if (!_btnId) {
				_btnId = Util.getAutoID();
				_btn.attr('id', _btnId);
			}
			if (hasTouch && e.originalEvent.touches.length > 1) {
				Util.log('Multi-touch is not supported.');
				if (TOUCHDOWN_TARGET) {
					$('#' + TOUCHDOWN_TARGET).removeClass('down');
					TOUCHDOWN_TARGET = null;
				}
				if (TOUCH_LONGTAP_TID) {
					clearTimeout(TOUCH_LONGTAP_TID);
					TOUCH_LONGTAP_TID = null;
				}
				return;
			}
			if (_eType === 'touchstart' || _eType === 'mousedown') {
				TOUCHDOWN_TARGET = _btnId;
				if (!_btn.hasClass('down')) _btn.addClass('down');
				if (_btn.attr('data-action')) {
					e.stopPropagation();
                    
                    if( e.originalEvent.touches ) {
                        start_point = {
                            x: e.originalEvent.touches [0].screenX,
                            y: e.originalEvent.touches [0].screenY
                        };
                    } else {
                        start_point = null;
                    }
                    
				} else if (_btn.attr('data-downAction')) {
					e.preventDefault();
					if (!Views.isAnyViewInTransition) {
						setTimeout(function() { triggerCallback(_btn.attr('data-downAction'), _btnId); }, 0);
					}
					if (!_btn.attr('data-upAction')) {
						_btn.removeClass('down');
						TOUCHDOWN_TARGET = null;
					}
				} else if (_btn.attr('data-longTapAction')) {
					e.preventDefault();
					if (TOUCH_LONGTAP_TID) clearTimeout(TOUCH_LONGTAP_TID);
					TOUCH_LONGTAP_TID = setTimeout((function(btnId) {
						return function() {
							TOUCHDOWN_TARGET = null;
							TOUCH_LONGTAP_TID = null;
							triggerCallback($('#' + btnId).removeClass('down').attr('data-longTapAction'), btnId);
						};
					})(_btnId), 2000);
				}
			// } else if (((_eType === 'touchmove' || _eType === 'mousemove') && Views.isAnyViewScrolling) || Util.inMotionTrack || _eType === 'touchcancel' || _eType === 'mouseout') {
			} else if ( (_eType === 'touchmove' || _eType === 'mousemove') || Util.inMotionTrack || _eType === 'touchcancel' || _eType === 'mouseout') {
				_btn.removeClass('down');

				if ( start_point ) {
                            
					move_point = e.originalEvent.touches[0];

					var _moveX = move_point.screenX - start_point.x,
						_moveY = move_point.screenY - start_point.y,
						_move_dis = Math.sqrt( _moveX*_moveX + _moveY*_moveY );
                            
					if( _move_dis > 10 ) {
						TOUCHDOWN_TARGET = null;
					}
				}

				if ( Views.isAnyViewScrolling ) {
					TOUCHDOWN_TARGET = null;
				}

				if (_btn.attr('data-upAction')) {
					e.preventDefault();
					if (!Views.isAnyViewInTransition) {
						setTimeout(function() { triggerCallback(_btn.attr('data-upAction'), _btnId); }, 0);
					}
				}
				if (TOUCH_LONGTAP_TID) {
					clearTimeout(TOUCH_LONGTAP_TID);
					TOUCH_LONGTAP_TID = null;
				}
			} else if (_eType === 'touchend' || _eType === 'mouseup') {
				if (TOUCHDOWN_TARGET === _btnId && !_btn.hasClass('active')) {
					_btn.removeClass('down');
					TOUCHDOWN_TARGET = null;
					if (_btn.attr('data-action')) {
						e.preventDefault();
						if (!Views.isAnyViewInTransition) {
							_btn.addClass('active');
							setTimeout(function() { 
								triggerCallback(_btn.attr('data-action'), _btnId, e); 
							}, 0);
							setTimeout(function(){
								_btn.removeClass('active');
							}, 500);
						}
					} else if (_btn.attr('data-upAction')) {
						e.preventDefault();
						if (!Views.isAnyViewInTransition) {
							setTimeout(function() { triggerCallback(_btn.attr('data-upAction'), _btnId); }, 0);
						}
					}
					if (TOUCH_LONGTAP_TID) {
						clearTimeout(TOUCH_LONGTAP_TID);
						TOUCH_LONGTAP_TID = null;
					}
				}
			}
		});

		document.addEventListener('touchmove', function(e) {
			e.preventDefault();
		}, false);
		// for Android only
		document.addEventListener('backbutton', function(e) {
            var currentView = StageManager.getCurrentView();
			//隐藏键盘
			if( AppManager.keyboard === 'open' ) {
				$('input').blur();$('select').blur();
				return;
			}
			if(mscroller) {
                if( currentView.options.tmpl === 'memberEditItemView' ) {
                    Callback.pageBack();
                    return;
                }
				mscroller.mobiscroll('hide');
				mscroller = '';
				return;
			}

			if( currentView.options.tmpl === 'mainView' ) {
				Views.mainView.onPageBack();
			} else {
				Callback.pageBack();
			}
		}, false);
		wndw.on(mobile_system === 'android' ? 'resize' : RESIZE_EVENT, function() {
			// Layout.adjustWindowSize(wndw.width(), wndw.height());
		});
	}
}) (jQuery);