// ------------------------------------------------------------------------
// TabbedPanelLayout
// 
Layout.TabbedPanelLayout = Class.extend(Class.create(), Layout.LayoutBase, {
	fxRegistry: {
		'left-right': {
			'out' : ['ui-stage-moveToLeft', 'ui-stage-moveToRight'],
			'in'  : ['ui-stage-moveFromRight', 'ui-stage-moveFromLeft']
		},
		'left-right-fade': {
			'out' : ['ui-stage-moveToLeftFade', 'ui-stage-moveToRightFade'],
			'in'  : ['ui-stage-moveFromRight', 'ui-stage-moveFromLeft']
		},
		'left-right-ease': {
			'out' : ['ui-stage-moveToLeftEase', 'ui-stage-moveToRightEase'],
			'in'  : ['ui-stage-moveFromRightEase', 'ui-stage-moveFromLeftEase']
		},
		'top-bottom': {
			'out' : ['ui-stage-moveToTop', 'ui-stage-moveToBottom'],
			'in'  : ['ui-stage-moveFromBottom', 'ui-stage-moveFromTop']
		},
		'top-bottom-fade': {
			'out' : ['ui-stage-moveToTopFade', 'ui-stage-moveToBottomFade'],
			'in'  : ['ui-stage-moveFromBottom', 'ui-stage-moveFromTop']
		},
		'top-bottom-ease': {
			'out' : ['ui-stage-moveToTopEase', 'ui-stage-moveToBottomEase'],
			'in'  : ['ui-stage-moveFromBottomEase', 'ui-stage-moveFromTopEase']
		},
		'fade-in-out': {
			'out' : ['transition-fade-out', 'transition-fade-out'],
			'in'  : ['transition-fade-in', 'transition-fade-in']
		}
	},

	// ctor
	initialize: function(ct, fx) {
		this.transitionClass = this.fxRegistry[fx ? fx : 'left-right'];
		this.tabViewStack = [];

		ct.html(Views.tmpl.tabbedPanelLayout());
		var stageTabs = ct.find('div.ui-stage-tab');
		this.jqStageTabs = [
			stageTabs.eq(0),
			stageTabs.eq(1)
		];
		this.currentStageTabIndex = 0;
		this.currentTabViewIndex = -1;
	},

	willHide: function() {
		if (this.currentTabViewIndex !== -1) {	
			var currentTabView = this.tabViewStack[this.currentTabViewIndex];
			currentTabView.willHide();	
		}
	},

	didHide: function() {
		if (this.currentTabViewIndex !== -1) {	
			var currentTabView = this.tabViewStack[this.currentTabViewIndex];
			currentTabView.didHide();	
		}
		this.tabViewStack = [];
		this.jqStageTabs = null;
		this.currentStageTabIndex = 0;
		this.currentTabViewIndex = -1;
	},

	onResize: function(w, h) {
		$.each(this.tabViewStack, function(_, tabView) {
			tabView.onResize(w, h);
		});
	},

	addTabView: function(tabView) {
		this.tabViewStack.push(tabView);
	},

	getCurrentTabView: function() {
		return this.currentTabViewIndex !== -1 ? this.tabViewStack[this.currentTabViewIndex] : null;
	},

	showTabView: function(tabIndex, transitionEndCallback) {
		if (tabIndex < 0 || tabIndex >= this.tabViewStack.length)
			return;

		var nextStageTabIndex = this.currentTabViewIndex === -1 ? 0 : (this.currentTabViewIndex === tabIndex ? this.currentStageTabIndex : (this.currentStageTabIndex+1)%2),
			jqStageTab = this.jqStageTabs[nextStageTabIndex],
			newTabView = this.tabViewStack[tabIndex];

		if (tabIndex === this.currentTabViewIndex) {
			if (newTabView.onRefresh) newTabView.onRefresh();
			if (transitionEndCallback) transitionEndCallback();
			return;
		}

		if (this.currentTabViewIndex === -1 || this.currentTabViewIndex === tabIndex) {
			this.currentTabViewIndex = tabIndex;

			newTabView.willShow(jqStageTab);
			newTabView.didShow(jqStageTab);

			if (transitionEndCallback) transitionEndCallback();
			return;
		}

		var currentTabView = this.tabViewStack[this.currentTabViewIndex],
			classIndex = tabIndex - this.currentTabViewIndex > 0 ? 0 : 1,
			jqStageTabOut = this.jqStageTabs[this.currentStageTabIndex],
			self = this;
		// put forward...
		this.currentTabViewIndex = tabIndex;
		this.currentStageTabIndex = nextStageTabIndex;

		currentTabView.willHide();
		newTabView.willShow(jqStageTab);

		Views.isAnyViewInTransition = true;
		AnimManager.two(jqStageTabOut, this.transitionClass['out'][classIndex],
			jqStageTab.removeClass('ui-hidden'), this.transitionClass['in'][classIndex],
			function() {
				Views.isAnyViewInTransition = false;

				jqStageTabOut.addClass('ui-hidden');
				currentTabView.didHide();
				jqStageTabOut.html('');
				newTabView.didShow(jqStageTab);

				if (transitionEndCallback) transitionEndCallback();
			}
		);
	},

	clearContent: function() {
		if (this.currentTabViewIndex !== -1) {	
			var currentTabView = this.tabViewStack[this.currentTabViewIndex];
			currentTabView.willHide();
			currentTabView.didHide();
		}
		$.each(this.jqStageTabs, function(_, tab) {
			tab.html('');
		});
		this.currentStageTabIndex = 0;
		this.currentTabViewIndex = -1;
	}
});


// ------------------------------------------------------------------------
// StackPanelLayout
// 
Layout.StackPanelLayout = Class.extend(Class.create(), Layout.TabbedPanelLayout, {
	pushStackView: function(stackView, transitionEndCallback) {
		if (this._checkIsInTransition()) return;
		this.tabViewStack.push(stackView);
		var index = this.tabViewStack.length - 1;
		Layout.StackPanelLayout.__super__().showTabView.call(this, index, transitionEndCallback);
	},

	popStackView: function(index, transitionEndCallback) {
        if (this.currentTabViewIndex <= 0)
            return;

        if (this._checkIsInTransition()) return;

        if( index > -1 ) {
            var len = this.tabViewStack.length, delNum = len-index-2;
            this.tabViewStack.splice(index+1, delNum);
            this.currentTabViewIndex -= delNum;
        }
        Layout.StackPanelLayout.__super__().showTabView.call(this, this.currentTabViewIndex - 1, transitionEndCallback);
        return this.tabViewStack.pop();
	},

	getCurrentStackView: function() {
		return this.currentTabViewIndex !== -1 ? this.tabViewStack[this.currentTabViewIndex] : null;
	},

    getPreviousStackView: function() {
        return this.currentTabViewIndex > 0 ? this.tabViewStack[this.tabViewStack.length - 2] : null;
    },

    getStackView: function(index) {
        if (index < 0) {
            return this.tabViewStack[this.tabViewStack.length + index];
        } else {
            return this.tabViewStack[index];
        }
    },

	getStackSize: function() {
		return this.tabViewStack.length;
	},

	fixViewStack: function(viewStack, callback) {
		var self = this;
		setTimeout(function() {
			if (!viewStack) {
				console.error('StackPanelLayout.fixViewStack >> the given viewStack is null.');
				return;
			}
			if (self.tabViewStack.last() !== viewStack.last()) {
				console.error('StackPanelLayout.fixViewStack >> the last view of the given viewStack does not match current view.');
				self.debugViewStack();
				return;
			}
			self.tabViewStack = viewStack;
			self.currentTabViewIndex = viewStack.length - 1;
			if (callback) callback();
		}, 0);  // yield some time to make console.error happy
	},

	debugViewStack: function() {
		var stack = [];
		$.each(this.tabViewStack, function(_, tabView) {
			var opt = tabView.options || {};
			stack.push(opt.id || opt.tmpl);
		});
		Util.log('Current view stack: [' + stack + ']');
	},
	
	// ----------------------------- Inherited but not implemented

	addTabView: function() {
		throw 'Not implemented';
	},

	getCurrentTabView: function() {
		throw 'Not implemented';
	},

	showTabView: function() {
		throw 'Not implemented';
	},
	_checkIsInTransition: function() {
		if (Views.isAnyViewInTransition) {
			Util.log('Some view is in transition...');
		}
		return Views.isAnyViewInTransition;
	}
});


// ------------------------------------------------------------------------
// SlideLayout
// 
Layout.SlideLayout = Class.extend(Class.create(), Layout.TabbedPanelLayout, {
	showNextSlideView: function(slideView, transitionEndCallback) {
		if (this.tabViewStack.length === 0) {
			this.tabViewStack.push(slideView);
			Layout.StackPanelLayout.__super__().showTabView.call(this, 0, transitionEndCallback);
			return;
		}
		this.tabViewStack = [this.getCurrentSlideView(), slideView];
		this.currentTabViewIndex = 0;
		Layout.StackPanelLayout.__super__().showTabView.call(this, 1, transitionEndCallback);
	},

	showPrevSlideView: function(slideView, transitionEndCallback) {
		if (this.tabViewStack.length === 0) {
			this.tabViewStack.push(slideView);
			Layout.StackPanelLayout.__super__().showTabView.call(this, 0, transitionEndCallback);
			return;
		}
		this.tabViewStack = [slideView, this.getCurrentSlideView()];
		this.currentTabViewIndex = 1;
		Layout.StackPanelLayout.__super__().showTabView.call(this, 0, transitionEndCallback);
	},

	getCurrentSlideView: function() {
		return this.currentTabViewIndex !== -1 ? this.tabViewStack[this.currentTabViewIndex] : null;
	},

	// ----------------------------- Inherited but not implemented

	addTabView: function() {
		throw 'Not implemented';
	},

	getCurrentTabView: function() {
		throw 'Not implemented';
	},

	showTabView: function() {
		throw 'Not implemented';
	}
});
Layout.SlideView = Class.extend(Class.create(), Views.PanelView, {
	initialize: function(options, delegate) {
		this.options = options || Views.PanelView.options;
		this.delegate = delegate;
	},

	willShow: function(ct) {
		if (this.delegate && this.delegate.willShow)
			this.delegate.willShow.call(this, ct);
	},

	didShow: function(ct) {
		if (this.delegate && this.delegate.didShow)
			this.delegate.didShow.call(this, ct);
		this.$super().didShow(ct);
	},

	willHide: function() {
		if (this.delegate && this.delegate.willHide)
			this.delegate.willHide.call(this);
		this.$super().willHide();
	},

	didHide: function() {
		if (this.delegate && this.delegate.didHide)
			this.delegate.didHide.call(this);
		this.$super().didHide();
	}
});
