// 在Callback上注册回调方法。
// 有下面几种方式会从Callback里调用回调方法：
// - 在一个带有.ui-btn的css class name的元素上点击，区分两种情况：
//   a. 如果想在鼠标点下去或者触摸屏上手指点上去的时候直接执行，在元素上加上data-downAction="<YourCallbackFunc>",
//      之后鼠标或者手指放开的时候会触发data-upAction
//   b. 如果想在鼠标释放或者触摸屏上手指移开才执行，在元素上加上 data-action="<YourCallbackFunc>"
//   c. 如果要长按触发某个时间，在元素上加上data-longTapAction="<YourCallbackFunc>"
// (参考 bindEvents 方法)
var Callback = {
    pageBackToView: function(pageName) {
        StageManager.popView(pageName);
    }
};

// 通用方法
var Util = {
	MILLIS_1_SEC: 1000,
	MILLIS_1_MIN: 60000,
	MILLIS_10_MIN: 600000,
	MILLIS_HALF_HOUR: 1800000,
	MILLIS_1_HOUR: 3600000,
	MILLIS_1_DAY: 86400000,

	inAppBrowserRef: null,
	debug: APPVERSION !== 'product',
	log: function(msg) {
		if (!this.debug)
			return;
		if (typeof msg === 'object')
			msg = JSON.stringify(msg);
		console.log('=====>>>>> ' + msg + ' <<<<<=====');
	},
	rtime: function() {
		return (Math.floor(Math.random() * 10001)).toString() + (new Date().getTime()).toString();
	},
	isOnline: function() {
		this.log('检测网络状态：' + window.networkState);
		return (window.networkState !== 'No network connection' && window.networkState !== 'Unknown connection');
	},
	networkStateChanged: function(networkState) {
		if (this.isOnline())
			return;
		// TODO
	},
	data: function(name, data) {
		if (typeof data !== 'undefined') {
			localStorage.setItem(name, JSON.stringify(data));
		} else {
			data = localStorage.getItem(name);
			return data ? JSON.parse(data) : undefined;
		}
	},
	alert: function(msg, callback, title, buttonName) {
		if (!mobile_system) {
			alert(msg);
			if (callback) callback();
			return;
		}
		var _title = title ? title : '温馨提示',
			_callback = callback ? callback : function() {};
		_buttonName = buttonName ? buttonName : '确定';
		navigator.notification.alert(msg, _callback, _title, _buttonName);
	},
    alertWindow: function(data){
        $('<div id="alertWindow" class="ui-overlay-fullscreen"></div>').appendTo($('body'));
        $('#alertWindow').html(Views.tmpl.alertWindowView({
            data: data
        }));
        StageManager.getCurrentView().makeScrollable($('#alertWindow .content'));
    },
	confirm: function(msg, callback, title, buttonLabels) {
		if (!mobile_system) {
			var _r = confirm(msg) ? 2 : 1;
			setTimeout(function() {
				callback(_r);
			}, 0);
			return;
		}
		var _title = title ? title : '请确认',
			_callback = callback ? callback : function(index) {};
		_buttonLabels = buttonLabels ? buttonLabels : '取消,确定';
		navigator.notification.confirm(msg, callback, _title, _buttonLabels);
	},
	beep: function(n) {
		if (!mobile_system && APPVERSION === 'web')
			return;
		var _n = n ? n : 2;
		navigator.notification.beep(_n);
		navigator.notification.vibrate(2000);
	},

	_autoID: 0,
	getAutoID: function() {
		this._autoID = this._autoID + 1;
		return '_id_' + this._autoID.toString();
	},

	ajaxFailureHandler: function(jqXHR, textStatus, errorThrown) {
		if (navigator.network.connection.type === Connection.NONE) {
			Util.alert('无网络连接，请检查您的设置！');
		} else {
			Util.alert('网络异常，请稍候重试...');
		}
	},

	// Update相关
	checkUpdateOrNewInstall: function(callback) {
		Util.log('Current version: ' + VERSION_NUMBER);
		var lastVersion = Util.data('version.lastVersion');
		if (!lastVersion) {
			Util.data('version.lastVersion', VERSION_NUMBER);
			if (callback) callback({ isNewInstall: true });
		} else if (lastVersion !== VERSION_NUMBER) {
			Util.data('version.lastVersion', VERSION_NUMBER);
			if (callback) callback({ isUpdate: true, lastVersion: lastVersion });
		} else {
			if (callback) callback({ isNewInstall: false, isUpdate: false });
		}
	},

	checkVersionUpdate: function(force) {

        if(CHANNEL_ID === 20002){ //百度市场
            cordova.exec(
                function(winParam) {},
                function(error) {

                },
                "BaiduUpdatePlugin",
                "baidu_update",
                []
            );
            return;
        }

//		if (APPVERSION === 'web') return;
		// 如果用户之前选择了'暂不更新'，1天内不再自动检查更新
		var deferUpdate = Util.data('version.deferUpdate');
		if (!force && deferUpdate && (new Date() - deferUpdate < Util.MILLIS_1_DAY))
			return;
		// 检查是否有更新
		if (force) {
			Widget.popupLoading('正在检查版本更新...');
		} else {
			Util.log('正在检查版本更新...');
		}
        FuckHttp.checkVersion({
            success: function(json) {
                if (force) Widget.popupLoadingClose();
                /*if (!json.success) {
                 if (force) Util.alert(json.errorMsg);
                 return;
                 }*/
                var version = json.data.versionID,
                    hasNewVersion = false,
                    v1 = version.split('.'), v2 = VERSION_NUMBER.split('.');
                for (var i = 0; i < 3; ++i) {
                    if (v1[i]-0 > v2[i]-0) {
                        LAST_VERSION_NUMBER = version;
                        hasNewVersion = true;
                        break;
                    }else if(v1[i]-0 < v2[i]-0){
                        break;
                    }

                }
                if (hasNewVersion) {
                    //更新页面和设置缓存
                    Util.data('hasNewVersion', true);

                    if( json.data.mustUpgrade === 'NO' ) {
                        Util.confirm('发现新版本' + version + ':  ' + (json.data.content || '') + '是否要立即更新？', function(index) {
                            if (index === 2) {
                                window.open(json.data.versionUrl, '_system');
                            } else {
                                Util.data('version.deferUpdate', new Date() - 0);
                            }
                        }, null, '暂不更新,更新');
                    } else {
                        Util.alert('有新版本' +LAST_VERSION_NUMBER + '需要更新:  ' + json.data.content || '', function() {
                            window.open(json.data.versionUrl, '_system');
                        }, null, '确定');
                        return;
                    }
                } else if (force) {
//                    Util.alert('当前已经是最新版本！');
                }
            },
            doneError: function(json){

            }
        })
	},

	// Make scrollable
	makeScrollable: function(jqScroller, options) {
        var scroll =  new iScroll(jqScroller[0], $.extend({
            checkDOMChanges: true,
            scrollbarClass: 'myScrollbar',
            // /*bounce: false,*/ // Setting this value to false to make it behave like in android
            // fadeScrollbar: true,
            // hideScrollbar: true,
            hScrollbar: false,
            vScrollbar: false,
            onBeforeScrollStart: function(e) {
                var target = e.target;
                while (target.nodeType !== 1)
                    target = target.parentNode;
                var _tagNameLc = target.tagName.toLowerCase();
                if (_tagNameLc !== 'select' && _tagNameLc !== 'input' && _tagNameLc !== 'textarea' && _tagNameLc !== 'option') {
                      $('textarea').blur();
                      $('input').blur();
                    e.preventDefault();
                }
//                    target.blur();
            },
            onScrollMove: function(e) {
                Views.isAnyViewScrolling = true;
                if(options && options.needScroll){
                    var currentView = StageManager.getCurrentView();
                    currentView.options.iScroll_Y = this.y;
                }
            },
            onScrollEnd: function(e) {
                Views.isAnyViewScrolling = false;
            },
            onTouchEnd: function() {
                var touchend = $(this.wrapper).attr('data-touchend');
                if (touchend) Callback[touchend]();
            }
        }, options || {}));

        if( options && options.needScroll ) {
            var StageManager = require('foundation.StageManager'),
                currentView = StageManager.getCurrentView();
            scroll.scrollTo(0, currentView.options.iScroll_Y || 0, 0);
        }

        return scroll;
    },

	makeScrollableWithPullRefresh: function(jqScroller, delegate, options) {
		return new iScroll(jqScroller[0], $.extend({
				checkDOMChanges: true,
				topOffset: 38,
				hScrollbar: false,
				vScrollbar: false,
				onRefresh: function () {
					var el = document.getElementById('pullDown'),
						el2 = document.getElementById('pullUp');
					if (el && el.className.match('ui-pulldown loading')) {
						el.className = 'ui-pulldown';
						el.querySelector('.pullDownLabel').innerHTML = '下拉立即刷新';
					}
					if (el2 && el2.className.match('ui-pullUp loading')) {
						el2.className = 'ui-pullUp';
						el2.querySelector('.pullup-inner').innerHTML = '向上加载更多';
					}
				},
				onBeforeScrollStart: function(e) {
					var target = e.target;
					while (target.nodeType !== 1)
						target = target.parentNode;
					var _tagNameLc = target.tagName.toLowerCase();
					if (_tagNameLc !== 'select' && _tagNameLc !== 'input' && _tagNameLc !== 'textarea' && _tagNameLc !== 'option')
						e.preventDefault();
				},
				onScrollMove: function () {
					Views.isAnyViewScrolling = true;
					var el = document.getElementById('pullDown'), el2 = document.getElementById('pullUp');
					if( isNaN(this.y) ) {
						el.className = 'ui-pulldown flip';
						el.querySelector('.pullDownLabel').innerHTML = '松开立即刷新';
						this.minScrollY = 0;
						return;
					}
					if (this.y > 5) {
						el.className = 'ui-pulldown flip';
						el.querySelector('.pullDownLabel').innerHTML = '松开立即刷新';
						this.minScrollY = 0;
					} else if (this.y > -38 && this.y < 5) {
						el.className = 'ui-pulldown';
						el.querySelector('.pullDownLabel').innerHTML = '下拉立即刷新';
						this.minScrollY = -38;
					} else if (el2 && this.y < this.wrapperH-this.scrollerH-38 && 
						!el2.className.match('ui-pullUp loading') && !el2.className.match('ui-pullUp ui-hidden')) {
						el2.className = 'ui-pullUp loading';
						el2.querySelector('.pullup-inner').innerHTML = '';
						if (delegate && delegate.onPullUpRefresh) delegate.onPullUpRefresh();
					}
				},
				onScrollEnd: function () {
					Views.isAnyViewScrolling = false;
					var el = document.getElementById('pullDown');
					if (el.className.match('ui-pulldown flip')) {
						el.className = 'ui-pulldown loading';
						el.querySelector('.pullDownLabel').innerHTML = '加载中...';				
						if (delegate && delegate.onPullDownRefresh) delegate.onPullDownRefresh();
					}
				}
			}, options || {}) // $.extend
		);
	},

	// 输入框清空按钮和事件绑定
	bindForInputClearBtn: function(els){
		els.on('focus input', function() {
			if (this.value)
				$(this).next('.input-empty').removeClass('ui-hidden');
			else
				$(this).next('.input-empty').addClass('ui-hidden');
		});
		els.on('blur', function(event){
			$(this).next('.input-empty').addClass('ui-hidden');
		});
	},
    //输入框输入验证
    inputs: function(els){
        els.on('focus', function(){
            if( this.value )
                $(this).next('.input-empty').removeClass('ui-hidden');
            else
                $(this).next('.input-empty').addClass('ui-hidden');
        });
        els.on('blur', function(event){
            $(this).next('.input-empty').addClass('ui-hidden');
        });
        els.on('input',function(){
            if( this.value )
                $(this).next('.input-empty').removeClass('ui-hidden');
            else
                $(this).next('.input-empty').addClass('ui-hidden');
        });
    }
};
Callback.emptyInput = function(btnId) {
	Util.log('emptyInput');
	$('#'+btnId).prev('input').val('').focus();
};


// ------------------------------------------------------------------------
// Validator
//
var Validator = {
	phoneNo: /^\d{8,11}$/,
	mobileNo: /^1[3|4|5|7|8][0-9]\d{8}$/,
	email: /^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/,
	realName: /^[\u4e00-\u9fa5]{2,5}$/,
	idCard: /(^\d{15}$)|(^\d{18}$)|(^\d{17}(\d|X|x)$)/,
	medicalCardNo:/^[0-9a-zA-Z]+$/,
	num_en: /^[0-9a-zA-Z]{6,16}$/,
	letter: /^[a-zA-Z]+$/,
	stringIsMobileNo: function(s) {
		return this.mobileNo.test(s);
	},
	stringContainsSpace: function(s) {
		return (/\s/).test(s);
	},
	stringIsNumOrEn:function (s) {
		return this.num_en.test(s);
	},
    stringIsRealName: function (s){
        return this.realName.test(s);
    },
    stringIsIdCard: function(s) {
        return this.idCard.test(s);
    },
    idTransfer: function(sId) {

		if( sId.length === 15 ) {
			var arrInt = new Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2),
				arrCh = new Array('1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2'),
				nTemp = 0, i;  

            sId = sId.substr(0, 6) + '19' + sId.substr(6, sId.length - 6);

			for(i = 0; i < 17; i ++) {
				nTemp += sId.substr(i, 1) * arrInt[i];
			}

            sId += arrCh[nTemp % 11];
		}

		return sId;
    },
    isChinaId: function(s) {
		var iSum = 0, info = "", sId = this.idTransfer(s),
			aCity = {
			11:"北京",12:"天津",13:"河北",14:"山西",15:"内蒙古",
			21:"辽宁",22:"吉林",23:"黑龙江 ",
			31:"上海",32:"江苏",33:"浙江",34:"安徽",35:"福建",36:"江西",37:"山东",
			41:"河南",42:"湖北 ",43:"湖南",44:"广东",45:"广西",46:"海南",
			50:"重庆",51:"四川",52:"贵州",53:"云南",54:"西藏 ",
			61:"陕西",62:"甘肃",63:"青海",64:"宁夏",65:"新疆",
			71:"台湾",
			81:"香港",82:"澳门",
			91:"国外 "
		};

		if ( !/^\d{17}(\d|x)$/i.test(sId) ) {
			return [false,'身份证格式错误'];
		}

		sId = sId.replace(/x$/i, "a");

		if ( !aCity[ parseInt( sId.substr(0, 2), 10 ) ] ) {
			return [false,'身份证前两位错误'];
		}

		var sBirthday = sId.substr(6, 4) + "-" + Number(sId.substr(10, 2)) + "-" + Number(sId.substr(12, 2)),
			d = new Date(sBirthday.replace(/-/g, "/"));

		if ( sBirthday !== (d.getFullYear() + "-" + (d.getMonth() + 1) + "-" + d.getDate())) {
			return [false,'非法身份证号'];
		}

		var sMouth = d.getMonth() + 1, sDate = d.getDate();
		sBirthday = d.getFullYear() + "-" + ( sMouth>9?sMouth:'0'+sMouth ) + "-" + ( sDate>9?sDate:'0'+sDate );

		/*十八位身份证的校验码*/
		/*for (var i = 17; i >= 0; i--) {
			iSum += (Math.pow(2, i) % 11) * parseInt(sId.charAt(17 - i), 11);
			if ( iSum % 11 !== 1 ) {
				return "Error:非法证号";
			} 
		}*/

		return [true, sBirthday, (sId.substr(16, 1) % 2 ? "MALE": "FEMALE")];
    }
};


// ------------------------------------------------------------------------
// Motion track extension for jQuery
//
(function($) {
	'use strict';

	var MOTION_TYPE_H  = 0,
		MOTION_TYPE_V  = 1,
		MOTION_TYPE_HV = 2,
		TRACK_XY_THRESHOLD = 8, // in px
		hasTouch = 'ontouchstart' in window,
		/* jslint smarttabs:true */
		TOUCH_EVENT = hasTouch ? 'touchstart.__mt touchend.__mt touchcancel.__mt touchmove.__mt'
							   : 'mousedown.__mt mouseup.__mt mousemove.__mt',
		defaultDelegate = {
			onTrackStart: function(target, pt) {},
			onTrackMove: function(target, pt) {},
			onTrackEnd: function(target) {}
		};

	$.fn.motionTrack = function(op, opts) {
		return this.each(function() {
			var el = $(this);
			if (op === 'start') {
				if (!opts.selector || typeof opts.selector !== 'string')
					return;
				startMotionTrack(el, getMoveType(opts.type), opts.selector, $.extend({}, defaultDelegate, opts.delegate));
			} else if (op === 'stop') {
				stopMotionTrack(el);
			}
		});
	};

	function getMoveType(str) {
		if (str === 'h')
			return MOTION_TYPE_H;
		else if (str === 'v')
			return MOTION_TYPE_V;
		else
			return MOTION_TYPE_HV;
	}

	function shouldStartTrack(motionType, dx, dy) {
		dx = Math.abs(dx);
		dy = Math.abs(dy);
		switch (motionType) {
		case MOTION_TYPE_H:
			return dx > TRACK_XY_THRESHOLD && dx > dy;
		case MOTION_TYPE_V:
			return dy > TRACK_XY_THRESHOLD && dy > dx;
		//case MOTION_TYPE_HV:
		default:
			return Math.abs(dx) > TRACK_XY_THRESHOLD || Math.abs(dy) > TRACK_XY_THRESHOLD;
		}
	}

	function startMotionTrack(el, motionType, selector, delegate) {
		var trackInitialized = false, trackStarted = false,
			originPt = null, target = null;

		el.on(TOUCH_EVENT, selector, function(e) {
			var _t = e.type, ev;

			if (_t === 'touchstart' || _t === 'mousedown') {
				if (target && target !== this && trackStarted) {
					delegate.onTrackEnd(target);
					trackStarted = false;
				}
				ev = hasTouch ? e.originalEvent.touches[0] : e;
				originPt = { x: ev.pageX, y: ev.pageY };
				trackInitialized = true;
				target = this;
			} else if (target === this && (_t === 'touchmove' || _t === 'mousemove')) {
				if (!trackInitialized)
					return;
				ev = hasTouch ? e.originalEvent.touches[0] : e;
				if (!trackStarted && shouldStartTrack(motionType, ev.pageX - originPt.x, ev.pageY - originPt.y)) {
					trackStarted = true;
					delegate.onTrackStart(target, originPt);
					Util.inMotionTrack = true;
				}
				if (trackStarted) {
					delegate.onTrackMove(target, { x: ev.pageX, y: ev.pageY });
					e.stopPropagation();
				}
			} else if (trackInitialized) { // _t === 'touchend' || _t === 'touchcancel' || _t === 'mouseup' || _t === 'mouseout'
				if (trackStarted) {
					delegate.onTrackEnd(target);
					//e.stopPropagation();
					setTimeout(function() {
						Util.inMotionTrack = false;
					}, 50);
				}
				trackInitialized = false;
				trackStarted = false;
				originPt = null;
				target = null;
			}
		});
	}

	function stopMotionTrack(el) {
		el.off(TOUCH_EVENT);
	}
}) (jQuery);


// ------------------------------------------------------------------------
// DateUtil
//
var DateUtil = {
	pad2digits: function(n) {
		return n < 10 ? '0' + n : '' + n;
	},

	formatDate: function(date) {
		return date.getUTCFullYear() + '-' + this.pad2digits(date.getUTCMonth()+1) + '-' + this.pad2digits(date.getUTCDate()) +
			' ' + this.pad2digits(date.getHours()) + ':' + this.pad2digits(date.getMinutes());
	},
	
	formatWithPatternDate : function (format, date) {
		if (!date)
			return null;
		var look = function (m) { // Check whether a format character is doubled
				var n = 0;
				while (i + 1 < format.length && format.charAt(i + 1) === m) {
					n++;
					i++;
				}
				return n;
			},
			f1 = function (m, val, len) { // Format a number, with leading zero if necessary
				var n = '' + val;
				if (look(m)) {
					while (n.length < len) {
						n = '0' + n;
					}
				}
				return n;
			},
			i,
			output = '',
			literal = false;

		for (i = 0; i < format.length; i++) {
			if (literal) {
				if (format.charAt(i) === "'" && !look("'")) {
					literal = false;
				} else {
					output += format.charAt(i);
				}
			} else {
				switch (format.charAt(i)) {
				case 'd':
					output += f1('d', date.getDate(), 2);
					break;
				case 'o':
					output += f1('o', (date.getTime() - new Date(date.getFullYear(), 0, 0).getTime()) / 86400000, 3);
					break;
				case 'm':
					output += f1('m', date.getMonth() + 1, 2);
					break;
				case 'y':
					output += (look('y') ? date.getFullYear() : (date.getYear() % 100 < 10 ? '0' : '') + date.getYear() % 100);
					break;
				case 'h':
					var h = date.getHours();
					output += f1('h', (h > 12 ? (h - 12) : (h === 0 ? 12 : h)), 2);
					break;
				case 'H':
					output += f1('H', date.getHours(), 2);
					break;
				case 'i':
					output += f1('i', date.getMinutes(), 2);
					break;
				case 's':
					output += f1('s', date.getSeconds(), 2);
					break;
				case 'a':
					output += date.getHours() > 11 ? '下午' : '上午';
					break;
				case 'A':
					output += date.getHours() > 11 ? 'PM' : 'AM';
					break;
				case "'":
					if (look("'")) {
						output += "'";
					} else {
						literal = true;
					}
					break;
				default:
					output += format.charAt(i);
				}
			}
		}
		return output;
	},

	isValidDate: function(dateStr) {
		var d;
		try {
			d = new Date(dateStr);
		} catch (e) {
			return false;
		}
		var now = new Date();
		/* jslint eqeqeq:true */
		if (!(d < now || d == now || d > now))
			return false;
		return true;
	}
};



// ------------------------------------------------------------------------
// String and Array utilities
//
(function() {
	'use strict';

	if (typeof String.prototype.startsWith === 'undefined') {
		String.prototype.startsWith = function(s) {
			if (!s) return false;
			return this.indexOf(s) === 0;
		};
	}

	if (typeof String.prototype.endsWith === 'undefined') {
		String.prototype.endsWith = function(s) {
			if (!s || !s.length) return false;
			return this.lastIndexOf(s) === this.length - s.length;
		};
	}

	if (typeof Array.prototype.insert === 'undefined') {
		Array.prototype.insert = function(index, item) {
			this.splice(index, 0, item);
			return true;
		};
	}

	if (typeof Array.prototype.remove === 'undefined') {
		Array.prototype.remove = function(index) {
			var arr = this.splice(index, 1);
			return (arr && arr.length === 1) ? arr[0] : null;
		};
	}

	if (typeof Array.prototype.first === 'undefined') {
		Array.prototype.first = function() {
			return this.length > 0 ? this[0] : null;
		};
	}

	if (typeof Array.prototype.last === 'undefined') {
		Array.prototype.last = function() {
			return this.length > 0 ? this[this.length - 1] : null;
		};
	}
}) ();


// ------------------------------------------------------------------------
// Other helper functions
//
function formSubmitTransfer(id) {
	$('input').blur();
	if (id && Callback[id]) Callback[id]();
	return false;
}

function setScrollPosition(scroll) {
	if (!scroll) {
		return;
	}
	Util.data('remind.position', scroll.y);
}
function getScrollPosition() {
	var y = Util.data('remind.position') || 0;
	Util.data('remind.position', '');
	return y;
}
function clearScrollPosition() {
	Util.data('remind.position', '');
}
Date.prototype.format = function(format)
{
    var o = {
        "M+" : this.getMonth()+1, //month
        "d+" : this.getDate(), //day
        "h+" : this.getHours(), //hour
        "m+" : this.getMinutes(), //minute
        "s+" : this.getSeconds(), //second
        "q+" : Math.floor((this.getMonth()+3)/3), //quarter
        "S" : this.getMilliseconds() //millisecond
    };
    if(/(y+)/.test(format))
        format=format.replace(RegExp.$1,(this.getFullYear()+"").substr(4 - RegExp.$1.length));
    for(var k in o)
        if(new RegExp("("+ k +")").test(format))
            format = format.replace(RegExp.$1,RegExp.$1.length===1 ? o[k] : ("00"+ o[k]).substr((""+ o[k]).length));
    return format;
};
